import org.apache.commons.lang3.StringUtils

import com.atlassian.jira.component.ComponentAccessor
import com.atlassian.jira.issue.link.IssueLinkTypeManager
import com.atlassian.jira.issue.views.util.IssueViewUtil

issueLinkTool = new IssueLinkTool(user: user)

public class IssueLinkTool {
	def issueLinkTypeManager = ComponentAccessor.getOSGiComponentInstanceOfType(IssueLinkTypeManager.class)
	def issueViewUtil = ComponentAccessor.getOSGiComponentInstanceOfType(IssueViewUtil.class)

	def user

	/**
	 * Whether to append "(in)" and "(out)" to the link names to denote the link direction in the export.
	 * It is particularly useful for link names that are identical in both directions (e.g. "relates to").
	 */
	boolean exportIssueLinkDirection = false

	def getIssueLinkTypes() {
		issueLinkTypeManager.getIssueLinkTypes()
	}

	def getInwardLinkName(linkType) {
		StringUtils.capitalize(linkType.inward) + (exportIssueLinkDirection ? " (in)" : "")
	}

	def getOutwardLinkName(linkType) {
		StringUtils.capitalize(linkType.outward) + (exportIssueLinkDirection ? " (out)" : "")
	}

	def getInwardLinkedIssues(issue, linkName) {
		getLinkedIssues(issueViewUtil.getLinkCollection(issue, user).inwardLinkMap, linkName)
	}

	def getOutwardLinkedIssues(issue, linkName) {
		getLinkedIssues(issueViewUtil.getLinkCollection(issue, user).outwardLinkMap, linkName)
	}

	private def getLinkedIssues(linkMap, linkName) {
		linkMap.get(linkName).collect { it.key }.sort().join(', ')
	}
}